@@ -63,7 +63,9 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   KRadioAction* radioAction;
 
   m_editUndo = KStdAction::undo(this, SLOT(slotEditUndo()), actionCollection);
+  m_editUndo->setEnabled(false);
   m_editRedo = KStdAction::redo(this, SLOT(slotEditRedo()), actionCollection);
+  m_editRedo->setEnabled(false);
   m_editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection);
   m_editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection);
   m_editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection);
@@ -128,7 +130,7 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   radioAction->setExclusiveGroup("tools");
   radioAction = new KRadioAction(i18n("Filled Rectangle"), QIconSet(loader->loadIcon("f_rect",KIcon::User)), 0, this, SLOT(slotFillRectangle()), actionCollection, "Tool_fill_rectangle");
   radioAction->setExclusiveGroup("tools");
-  radioAction = new KRadioAction(i18n("OutlineEllipse"), QIconSet(loader->loadIcon("o_ellipse",KIcon::User)), 0, this, SLOT(slotEllipse()), actionCollection, "Tool_ellipse");
+  radioAction = new KRadioAction(i18n("Outline Ellipse"), QIconSet(loader->loadIcon("o_ellipse",KIcon::User)), 0, this, SLOT(slotEllipse()), actionCollection, "Tool_ellipse");
   radioAction->setExclusiveGroup("tools");
   radioAction = new KRadioAction(i18n("Filled Ellipse"), QIconSet(loader->loadIcon("f_ellipse",KIcon::User)), 0, this, SLOT(slotFillEllipse()), actionCollection, "Tool_fill_ellipse");
   radioAction->setExclusiveGroup("tools");
@@ -163,6 +165,8 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   radioAction->setExclusiveGroup("StitchView");
   radioAction = new KRadioAction(i18n("Colored blocks with symbols"), 0, this, SLOT(slotColorBlocksSymbols()), actionCollection, "colorblocksymbols");
   radioAction->setExclusiveGroup("StitchView");
+  radioAction = new KRadioAction(i18n("Color Highlight"), 0, this, SLOT(slotColorHighlight()), actionCollection, "colorhighlight");
+  radioAction->setExclusiveGroup("StitchView");
 
   new KToggleAction(i18n("Stitch Mask"), 0, this, SLOT(slotStitchMask()), actionCollection, "StitchMask");
   new KToggleAction(i18n("Color Mask"), 0, this, SLOT(slotColorMask()), actionCollection, "ColorMask");
@@ -198,6 +202,9 @@ EditView::EditView(KXStitchDoc* doc,QWidget* parent)
   connect(this,SIGNAL(contentsMoving(int,int)),this,SLOT(slotScrolling(int,int)));
   connect(&m_selectionTimer,SIGNAL(timeout()),this,SLOT(slotSelectionTimedOut()));
   emit selectionMade(false);
+
+  connect(doc,SIGNAL(undoStackNotEmpty(bool)),m_editUndo,SLOT(setEnabled(bool)));
+  connect(doc,SIGNAL(redoStackNotEmpty(bool)),m_editRedo,SLOT(setEnabled(bool)));
 }
 
 void EditView::setBackgroundImage(QImage image)
@@ -844,13 +851,13 @@ void EditView::configure(bool init)
 QRect EditView::selectionArea()
 {
   int left = m_selectedArea.left();
-  left = left>?0;
+  left = std::max(left,0);
   int top = m_selectedArea.top();
-  top = top>?0;
+  top = std::max(top,0);
   int right = m_selectedArea.right();
-  right = right<?contentsWidth();
+  right = std::min(right,contentsWidth());
   int bottom = m_selectedArea.bottom();
-  bottom = bottom <? contentsHeight();
+  bottom = std::min(bottom,contentsHeight());
   int width = (right-left+1)/m_cellSize;
   int height = (bottom-top+1)/m_cellSize;
   left /= m_cellSize;
@@ -888,12 +895,12 @@ int EditView::print(KPrinter* printer, QPainter* painter, int width, int height,
     if (instructions.isEmpty())
       painter->drawText(0,y,i18n("No additional instructions have been specified."));
     else
-      painter->drawText(0,y,width,height-y,Qt::WordBreak,instructions);
+      painter->drawText(0,y-dy,width,height-y-dy,Qt::WordBreak,instructions);
     /** TODO
         There is the potential for the instructions to span more than one page
         This should be checked for and page breaks added accordingly.
       */
-    painter->drawText(0,height+2*dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(++pages));
+    painter->drawText(0,height-dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(++pages));
     printer->newPage();
   }
 
@@ -980,13 +987,13 @@ int EditView::print(KPrinter* printer, QPainter* painter, int width, int height,
         painter->fillRect(b,QBrush(Qt::black));
         painter->fillRect(r,QBrush(Qt::white));
         painter->drawRect(r);
-        QRect currentPage(r.left()+(50*x/patternHeight),r.top()+(50*y/patternHeight),(50*pageCellsWide/patternHeight)<?(50*(patternWidth-x)/patternHeight),(50*pageCellsTall/patternHeight)<?(50*(patternHeight-y)/patternHeight));
-        painter->fillRect(currentPage,QBrush(Qt::lightGray));
+        QRect currentPage(r.left()+(50*x/patternHeight),r.top()+(50*y/patternHeight),std::min((50*pageCellsWide/patternHeight),(50*(patternWidth-x)/patternHeight)),std::min((50*pageCellsTall/patternHeight),(50*(patternHeight-y)/patternHeight)));
+        painter->fillRect(currentPage,QBrush(QColor(lightGray)));
         painter->drawRect(currentPage);
       }
-      printContents(printer, painter, x, y, printable, pageCellsWide<?(patternWidth-x), pageCellsTall<?(patternHeight-y), printCellSize);
+      printContents(printer, painter, x, y, printable, std::min(pageCellsWide,(patternWidth-x)), std::min(pageCellsTall,(patternHeight-y)), printCellSize);
       // print footer
-      painter->drawText(0,height+dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(++pages));
+      painter->drawText(0,height-dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(++pages));
       if (--totalPages)
         printer->newPage();
       painter->restore();
@@ -1299,6 +1306,19 @@ void EditView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cl
           doubleBuffer->drawText(r, AlignCenter, s, 1);
           doubleBuffer->drawEllipse(r);
           break;
+        case ColorHighlight:
+          if (k->floss == pPalette->currentColor())
+          {
+            doubleBuffer->setBrush(QBrush(c));
+            doubleBuffer->setPen(c);
+          }
+          else
+          {
+            doubleBuffer->setPen(QColor(lightGray));
+            doubleBuffer->setBrush(QBrush(QColor(lightGray)));
+          }
+          doubleBuffer->drawEllipse(r);
+          break;
         default:
           // no other values are relevant
           break;
@@ -1314,6 +1334,10 @@ void EditView::drawContents(QPainter *p, int clipx, int clipy, int clipw, int cl
     for ( ; (pBackstitch = it->current()) ; ++(*it) )
     {
       doubleBuffer->setPen(QPen(pPalette->flossAt(pBackstitch->floss)->color,3));
+      if (m_vmode == ColorHighlight && pBackstitch->floss != pPalette->currentColor())
+      {
+        doubleBuffer->setPen(QPen(QColor(lightGray)));
+      }
       doubleBuffer->drawLine(snapToContents(pBackstitch->start), snapToContents(pBackstitch->end));
     }
     delete it;
@@ -1782,8 +1806,11 @@ void EditView::contentsMouseMoveEvent(QMouseEvent *e)
       }
       break;
     case Paste:
-      repaintContents(QRect(cellToRect(m_end).topLeft(),QSize(m_pasteImage->size())));
-      bitBlt(viewport(),contentsToViewport(cellToRect(m_tracking).topLeft()),m_pasteImage,QRect(0,0,m_pasteImage->width(),m_pasteImage->height()),NotXorROP);
+      if ( m_pasteImage )
+      {
+        repaintContents(QRect(cellToRect(m_end).topLeft(),QSize(m_pasteImage->size())));
+        bitBlt(viewport(),contentsToViewport(cellToRect(m_tracking).topLeft()),m_pasteImage,QRect(0,0,m_pasteImage->width(),m_pasteImage->height()),NotXorROP);
+      }
       m_end = m_tracking;
       break;
   }
@@ -1860,7 +1887,7 @@ void EditView::contentsMouseReleaseEvent(QMouseEvent *e)
                 m_end = snapToContents(bs->end);
                 // found a backstitch that starts and ends where indicated
                 m_doc->deleteBackstitch(bs); // this should also update the iterator
-                updateContents(QRect(QPoint((m_start.x()<?m_end.x())-2,(m_start.y()<?m_end.y())-2),QPoint((m_start.x()>?m_end.x())+2,(m_start.y()>?m_end.y())+2)));
+                updateContents(QRect(QPoint((std::min(m_start.x(),m_end.x()))-2,(std::min(m_start.y(),m_end.y()))-2),QPoint((std::max(m_start.x(),m_end.x()))+2,(std::max(m_start.y(),m_end.y()))+2)));
                 break;
               }
             }
@@ -1912,7 +1939,7 @@ void EditView::contentsMouseReleaseEvent(QMouseEvent *e)
           drawBoundingRect(m_start, m_end);
           painter.begin(&dm);
           painter.setPen(QPen(black,1));
-          painter.drawRect(QRect(QPoint(m_start.x()<?m_end.x(),m_start.y()<?m_end.y()),QPoint(m_start.x()>?m_end.x(),m_start.y()>?m_end.y())));
+          painter.drawRect(QRect(QPoint(std::min(m_start.x(),m_end.x()),std::min(m_start.y(),m_end.y())),QPoint(std::max(m_start.x(),m_end.x()),std::max(m_start.y(),m_end.y()))));
           painter.end();
           m_doc->addStitches(processPixmap(dm),m_currentStitchType);
           emit locationOrSize(QPoint(-1,-1));
@@ -1923,7 +1950,7 @@ void EditView::contentsMouseReleaseEvent(QMouseEvent *e)
           painter.begin(&dm);
           painter.setPen(QPen(black,1));
           painter.setBrush(black);
-          painter.drawRect(QRect(QPoint(m_start.x()<?m_end.x(),m_start.y()<?m_end.y()),QPoint(m_start.x()>?m_end.x(),m_start.y()>?m_end.y())));
+          painter.drawRect(QRect(QPoint(std::min(m_start.x(),m_end.x()),std::min(m_start.y(),m_end.y())),QPoint(std::max(m_start.x(),m_end.x()),std::max(m_start.y(),m_end.y()))));
           painter.end();
           m_doc->addStitches(processPixmap(dm),m_currentStitchType);
           emit locationOrSize(QPoint(-1,-1));
@@ -1933,7 +1960,7 @@ void EditView::contentsMouseReleaseEvent(QMouseEvent *e)
           drawBoundingEllipse(m_start, m_end);
           painter.begin(&dm);
           painter.setPen(QPen(black,1));
-          painter.drawEllipse(QRect(QPoint(m_start.x()<?m_end.x(),m_start.y()<?m_end.y()),QPoint(m_start.x()>?m_end.x(),m_start.y()>?m_end.y())));
+          painter.drawEllipse(QRect(QPoint(std::min(m_start.x(),m_end.x()),std::min(m_start.y(),m_end.y())),QPoint(std::max(m_start.x(),m_end.x()),std::max(m_start.y(),m_end.y()))));
           painter.end();
           m_doc->addStitches(processPixmap(dm),m_currentStitchType);
           emit locationOrSize(QPoint(-1,-1));
@@ -1944,7 +1971,7 @@ void EditView::contentsMouseReleaseEvent(QMouseEvent *e)
           painter.begin(&dm);
           painter.setPen(QPen(black,1));
           painter.setBrush(black);
-          painter.drawEllipse(QRect(QPoint(m_start.x()<?m_end.x(),m_start.y()<?m_end.y()),QPoint(m_start.x()>?m_end.x(),m_start.y()>?m_end.y())));
+          painter.drawEllipse(QRect(QPoint(std::min(m_start.x(),m_end.x()),std::min(m_start.y(),m_end.y())),QPoint(std::max(m_start.x(),m_end.x()),std::max(m_start.y(),m_end.y()))));
           painter.end();
           m_doc->addStitches(processPixmap(dm),m_currentStitchType);
           emit locationOrSize(QPoint(-1,-1));
@@ -2067,7 +2094,7 @@ QRect EditView::visibleCells()
   int height=m_doc->canvas()->patternHeight();
   QPoint tl = contentsToCell(QPoint(contentsX(), contentsY()));
   QPoint br = contentsToCell(QPoint(contentsX()+visibleWidth(), contentsY()+visibleHeight()));
-  cells.setCoords(tl.x(), tl.y(), br.x()<?width, br.y()<?height);
+  cells.setCoords(tl.x(), tl.y(), std::min(br.x(), width), std::min(br.y(), height));
   return cells;
 }
 
@@ -2341,6 +2368,11 @@ void EditView::slotColorBlocksSymbols()
   setViewMode(ColorBlocksSymbols);
 }
 
+void EditView::slotColorHighlight()
+{
+  setViewMode(ColorHighlight);
+}
+
 void EditView::slotStitchMask()
 {
   m_stitchMask = !m_stitchMask;
@@ -2379,7 +2411,7 @@ void EditView::slotFitHeight()
 
 void EditView::slotFitPage()
 {
-  m_cellSize = (visibleWidth()/m_doc->canvas()->patternWidth())<?(visibleHeight()/m_doc->canvas()->patternHeight());
+  m_cellSize = std::min((visibleWidth()/m_doc->canvas()->patternWidth()),(visibleHeight()/m_doc->canvas()->patternHeight()));
   resizeGrid();
 }
 
@@ -2490,6 +2522,10 @@ void EditView::setMode(ToolMode m)
     m_pointArray.resize(0);
     m_points = 0;
   }
+  if (m_mode == Paste)
+  {
+    m_pasteImage = NULL;
+  }
 }
 
 void EditView::setViewMode(ViewMode m)
@@ -2677,6 +2713,13 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
         p->setBrush(c);
       QChar s(pPalette->symbolAt(pStitch->floss));
       if (viewMode!=BWSymbols) p->setPen(QPen(c, 2)); // this will be set from the stitch color index
+      if (viewMode == ColorHighlight)
+      {
+        if (pPalette->currentColor() != pStitch->floss)
+        {
+          p->setPen(QColor(lightGray));
+        }
+      }
       switch (cst)
       {
         case Stitch::TLQtr:
@@ -2684,6 +2727,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(0,0,cx,cy);
               break;
             case BWSymbols:
@@ -2711,6 +2755,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x1,y,cx,cy);
               break;
             case BWSymbols:
@@ -2738,6 +2783,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x,y1,cx,cy);
               break;
             case BWSymbols:
@@ -2765,6 +2811,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x1,y1,cx,cy);
               break;
             case BWSymbols:
@@ -2792,6 +2839,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x1,y,x,y1);
               break;
             case BWSymbols:
@@ -2819,6 +2867,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x,y,x1,y1);
               break;
             case BWSymbols:
@@ -2846,6 +2895,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x1,y,x,y1);
               p->drawLine(x,y,cx,cy);
               break;
@@ -2874,6 +2924,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x,y,x1,y1);
               p->drawLine(x1,y,cx,cy);
               break;
@@ -2902,6 +2953,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x,y,x1,y1);
               p->drawLine(x,y1,cx,cy);
               break;
@@ -2930,6 +2982,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x1,y,x,y1);
               p->drawLine(x1,y1,cx,cy);
               break;
@@ -2958,6 +3011,7 @@ void EditView::renderCell(QPainter *p, Stitch::Queue *pStitches,ViewMode viewMod
           switch(viewMode)
           {
             case Regular :
+	    case ColorHighlight :
               p->drawLine(x,y,x1,y1);
               p->drawLine(x1,y,x,y1);
               break;
