@@ -12,60 +12,18 @@
 #include <qptrlist.h>
 #include <kmessagebox.h>
 #include "flosslistboxitem.h"
-#include "addcolordialog.h"
-#include "deletecolordialog.h"
 #include "flosspalette.h"
 #include "flossscheme.h"
 #include "schememanager.h"
 #include "kxstitch.h"
+#include "palettemanagerdialog.h"
+#include "configuration.h"
 
 FlossPalette::FlossPalette()
   : m_current(-1)
 {
 }
 
-bool FlossPalette::addColors()
-{
-  bool added;
-  FlossListBoxItem *pFlossItem;
-  AddColorDialog *pDlg;
-  FlossScheme *pScheme = ((KXStitchApplication*)kapp)->scheme(m_schemeName);
-
-  added=false;
-  if (pScheme)
-  {                                               // make sure that pScheme actually points to a FlossScheme
-    pDlg = new AddColorDialog(0,m_schemeName);    // create an instance of the AddColorDialog
-    if (pDlg)
-    {                                             // and check that it was created successfully
-      QPtrListIterator<Floss> it = pScheme->flossIterator();
-      Floss *f;
-      for ( ; (f = it.current()) ; ++it)
-      {
-        if (!contains(f->name))                   // ignore any already in the palette
-          pFlossItem = new FlossListBoxItem(pDlg->ColorList, f);
-      }
-      if (pDlg->exec() == QDialog::Accepted)
-      {                                           // process the dialog if the user selected OK
-        int count = pDlg->ColorList->count();
-        for (int i = 0 ; i < count ; i++)
-        {
-          if (pDlg->ColorList->isSelected(i))
-          {                                       // isSelected is true if the QListBoxItem was selected in the QListBox
-                                                  // get the floss details, create a new Floss from it and add it to the palette
-            pFlossItem = (FlossListBoxItem *)(pDlg->ColorList->item(i));
-            int mi = freeIndex();
-            m_palette[mi] = pFlossItem->floss;
-            m_symbols[mi] = freeSymbol();
-            added=true;                           // indicate that colors were added, this is used by the app to update the paletteview
-          }
-        }
-      }
-      delete pDlg;                                // clean up
-    }
-  }
-  return added;
-}
-
 int FlossPalette::addColor(QColor color)
 {
   int i;
@@ -80,6 +38,8 @@ int FlossPalette::addColor(QColor color)
         int mi = freeIndex();
         m_palette[mi] = f;
         m_symbols[mi] = freeSymbol();
+        m_stitchStrands[mi] = KXSConfig().Palette_StitchStrands;
+        m_backstitchStrands[mi] = KXSConfig().Palette_BackstitchStrands;
         return mi;
       }
     }
@@ -87,44 +47,6 @@ int FlossPalette::addColor(QColor color)
   return i;
 }
 
-bool FlossPalette::removeColors(QValueList<int> usedColors)
-{
-  bool removed = false;
-  FlossListBoxItem *pFlossItem;
-  DeleteColorDialog *pDlg;
-  QMapIterator<int,Floss*> it;
-
-  pDlg = new DeleteColorDialog(0);             // create an instance of the AddColorDlg
-  if (pDlg)
-  {                                             // and check that it was created successfully
-    for (it = m_palette.begin() ; it != m_palette.end() ; ++it)
-    {
-      pFlossItem = new FlossListBoxItem(pDlg->ColorList, it.data());
-      if (usedColors.contains(it.key()))       // if the color has been used in the pattern
-        pFlossItem->setSelectable(false);
-    }
-    if (pDlg->exec() == QDialog::Accepted)
-    {                                         // process the dialog if the user selected OK
-      int count = pDlg->ColorList->count();
-      for (int i = 0 ; i < count ; i++)
-      {
-        if (pDlg->ColorList->isSelected(i))
-        {                                     // isSelected is true if the QListBoxItem was selected in the QListBox
-                                                // get the floss details and remove it from the palette
-          pFlossItem = (FlossListBoxItem *)(pDlg->ColorList->item(i));
-          int mi = contains(pFlossItem->floss->color);
-          if (mi == m_current) m_current = -1;
-          m_palette.remove(mi);
-          m_symbols.remove(mi);
-          removed=true;                         // indicate that colors were removed, this is used by the app to update the paletteview
-        }
-      }
-    }
-    delete pDlg;                                // clean up
-  }
-  return removed;
-}
-
 bool FlossPalette::removeUnused(QValueList<int> usedColors)
 {
   bool removed = false;
@@ -138,6 +60,8 @@ bool FlossPalette::removeUnused(QValueList<int> usedColors)
       if (mi == m_current) m_current = -1;
       m_palette.remove(mi);
       m_symbols.remove(mi);
+      m_stitchStrands.remove(mi);
+      m_backstitchStrands.remove(mi);
       removed=true;                         // indicate that colors were removed, this is used by the app to update the paletteview
     }
   }
@@ -229,6 +153,8 @@ void FlossPalette::setScheme(QString s)
   }
   m_palette.clear();
   m_symbols.clear();
+  m_stitchStrands.clear();
+  m_backstitchStrands.clear();
   m_current = -1;
 }
 
@@ -268,6 +194,9 @@ bool FlossPalette::changeScheme(QString name)
 void FlossPalette::clear()
 {
   m_palette.clear();
+  m_symbols.clear();
+  m_stitchStrands.clear();
+  m_backstitchStrands.clear();
   m_current = -1;
 }
 
@@ -281,6 +210,16 @@ QChar FlossPalette::symbolAt(int i)
   return m_symbols[i];
 }
 
+int FlossPalette::stitchStrandsAt(int i)
+{
+  return m_stitchStrands[i];
+}
+
+int FlossPalette::backstitchStrandsAt(int i)
+{
+  return m_backstitchStrands[i];
+}
+
 bool FlossPalette::loadPalette(QString schemeName)
 {
   FlossScheme* scheme = ((KXStitchApplication*)kapp)->scheme(schemeName);
@@ -293,6 +232,8 @@ bool FlossPalette::loadPalette(QString schemeName)
       int i = freeIndex();
       m_palette[i] = floss;
       m_symbols[i] = freeSymbol();
+      m_stitchStrands[i] = KXSConfig().Palette_StitchStrands;
+      m_backstitchStrands[i] = KXSConfig().Palette_BackstitchStrands;
     }
     return true;
   }
@@ -331,7 +272,10 @@ bool FlossPalette::writePalette(QDataStream &s)
   QMapIterator<int,Floss*> flosses;
   for (flosses = m_palette.begin() ; flosses != m_palette.end() ; ++flosses)
   {
-    s << (Q_INT32)flosses.key() << flosses.data()->name << (Q_INT16)m_symbols[flosses.key()].unicode(); // Q_INT32, QString, Q_INT16
+    s << (Q_INT32)flosses.key() << flosses.data()->name
+      << (Q_INT16)m_symbols[flosses.key()].unicode()
+      << (Q_INT8)m_stitchStrands[flosses.key()]
+      << (Q_INT8)m_backstitchStrands[flosses.key()];
   }
   return true;
 }
@@ -408,9 +352,33 @@ bool FlossPalette::readKXStitchPalette(QDataStream& s,int fileFormatVersion)
   QString       description;
   QColor        color;
   Q_INT16       symbol;
+  Q_INT8        stitchStrands;
+  Q_INT8        backstitchStrands;
   FlossScheme*  scheme;
   switch (fileFormatVersion)
   {
+    case 6:
+      s >> schemeName;
+      setScheme(schemeName);
+      scheme = ((KXStitchApplication *)kapp)->scheme(schemeName);
+      s >> current;
+      m_current = current;
+      s >> colors;
+      while (colors--)
+      {
+        s >> current;
+        s >> colorName;
+        s >> symbol;
+        s >> stitchStrands;
+        s >> backstitchStrands;
+        m_palette[current] = scheme->find(colorName);
+        if (m_palette[current] == 0)
+          return false;
+        m_symbols[current] = QChar((ushort)symbol);
+        m_stitchStrands[current] = stitchStrands;
+        m_backstitchStrands[current] = backstitchStrands;
+      }
+      break;
     case 5: // no changes from version 4 for this section
     case 4: // no changes from version 3 for this section
     case 3:
@@ -429,6 +397,8 @@ bool FlossPalette::readKXStitchPalette(QDataStream& s,int fileFormatVersion)
         if (m_palette[current] == 0)
           return false;
         m_symbols[current] = QChar((ushort)symbol);
+        m_stitchStrands[current] = KXSConfig().Palette_StitchStrands;
+        m_backstitchStrands[current] = KXSConfig().Palette_BackstitchStrands;
       }
       break;
     case 2:
@@ -460,6 +430,8 @@ bool FlossPalette::readKXStitchPalette(QDataStream& s,int fileFormatVersion)
         if (m_palette[current] == 0)
           return false;
         m_symbols[current] = QChar(symbol);
+        m_stitchStrands[current] = KXSConfig().Palette_StitchStrands;
+        m_backstitchStrands[current] = KXSConfig().Palette_BackstitchStrands;
       }
       break;
     default:
