@@ -21,18 +21,24 @@
 #include <kapplication.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
+#include <kglobal.h>
+#include <kiconloader.h>
 #include <qevent.h>
 #include "configuration.h"
 #include "importimagedlg.h"
 #include "kxstitch.h"
 #include "flossscheme.h"
 #include "floss.h"
+#include "alphaselect.h"
+
+#include <kdebug.h>
 
 ImportImageDlg::ImportImageDlg(const Magick::Image& image, QWidget *parent, const char *name, bool modal)
   : ImportImage(parent,name,modal),
     m_pixmap(new QPixmap),
     m_useFractionals(false),
-    m_orig(image)
+    m_orig(image),
+    m_ignoreColor(false)
 {
   m_originalSize = QSize(m_orig.columns(),m_orig.rows());
   QString caption = QString("Import Image - Image Size %1 x %2 pixels").arg(m_originalSize.width()).arg(m_originalSize.height());
@@ -47,7 +53,7 @@ ImportImageDlg::ImportImageDlg(const Magick::Image& image, QWidget *parent, cons
   m_preferredSize = QSize(KXSConfig().NewPattern_Width,KXSConfig().NewPattern_Height);
   int sw = m_preferredSize.width()*100/m_originalSize.width();
   int sh = m_preferredSize.height()*100/m_originalSize.height();
-  int s = sw <? sh; // pick smallest scale
+  int s = std::min(sw, sh); // pick smallest scale
 
   Import_FlossScheme->insertStringList(((KXStitchApplication*)kapp)->schemes());
   Configuration::setComboBoxItem(Import_FlossScheme,KXSConfig().NewPattern_FlossScheme);
@@ -58,6 +64,7 @@ ImportImageDlg::ImportImageDlg(const Magick::Image& image, QWidget *parent, cons
   connect(Import_FlossScheme, SIGNAL(activated(int)), this, SLOT(schemeChanged(int)));
   connect(Import_UseMaxColors, SIGNAL(stateChanged(int)), this, SLOT(maxColorsChanged(int)));
   connect(Import_MaxColors, SIGNAL(valueChanged(int)), this, SLOT(maxColorsChanged(int)));
+  connect(Import_ColorPicker, SIGNAL(clicked()), this, SLOT(pickColor()));
   connect(Import_ClothCount, SIGNAL(valueChanged(double)), this, SLOT(clothCountChanged(double)));
   connect(Import_PatternScale, SIGNAL(valueChanged(int)), this, SLOT(scaleChanged(int)));
   connect(Import_UseFractionals, SIGNAL(stateChanged(int)), this, SLOT(useFractionalsChanged(int)));
@@ -143,6 +150,7 @@ void ImportImageDlg::createImageMap()
 
 void ImportImageDlg::renderPixmap()
 {
+  QPixmap alpha = KGlobal::iconLoader()->loadIcon("alpha",KIcon::User);
   CropImageLabel->setCursor(waitCursor);
   calcSizes();
   m_image.modifyImage();
@@ -160,6 +168,7 @@ void ImportImageDlg::renderPixmap()
 
   QPainter p;
   p.begin(m_pixmap);
+  p.drawTiledPixmap(m_pixmap->rect(),alpha);
   int w = m_image.columns();
   int h = m_image.rows();
   int pixelCount = w*h;
@@ -168,7 +177,7 @@ void ImportImageDlg::renderPixmap()
   progress.setMinimumDuration(100);
 
   Magick::Pixels cache(m_image);
-  const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
+  const Magick::PixelPacket *pixels = cache.getConst(0,0,w,h);
   for (int cy = 0 ; cy < h ; cy++)
   {
     kapp->processEvents();
@@ -177,12 +186,14 @@ void ImportImageDlg::renderPixmap()
     for (int cx = 0 ; cx < w ; cx++)
     {
       Magick::PixelPacket packet = *pixels++;
-      Magick::Color c = Magick::Color(packet.red, packet.green, packet.blue);
       if (!(packet.opacity))
       {
-        QColor c(packet.red/257, packet.green/257, packet.blue/257);
-        p.setPen(QPen(c));
-        p.drawPoint(cx,cy);
+        if (!(m_ignoreColor && packet == m_ignoreColorValue))
+        {
+          QColor c(packet.red/257, packet.green/257, packet.blue/257);
+          p.setPen(QPen(c));
+          p.drawPoint(cx,cy);
+        }
       }
     }
   }
@@ -211,4 +222,36 @@ void ImportImageDlg::timerEvent(QTimerEvent *e)
 {
   killTimers();
   renderPixmap();
-}
\ No newline at end of file
+}
+
+void ImportImageDlg::pickColor()
+{
+  m_ignoreColor = Import_IgnoreColor->isChecked();
+  kdDebug() << "In ImportImageDlg::pickColor()" << endl;
+  if (m_ignoreColor)
+  {
+    m_alphaSelect = new AlphaSelect(CropImageLabel);
+    connect(m_alphaSelect,SIGNAL(clicked(QPoint)),this,SLOT(selectColor(QPoint)));
+    m_alphaSelect->show();
+  }
+  else
+  {
+    delete m_alphaSelect;
+    m_alphaSelect = 0;
+  }
+}
+
+void ImportImageDlg::selectColor(QPoint p)
+{
+  delete m_alphaSelect;
+  m_alphaSelect = 0;
+  QPixmap swatch(Import_IgnoreColorValue->size());
+  int x = p.x()-((CropImageLabel->width()-m_pixmap->width())/2);
+  int y = p.y()-((CropImageLabel->height()-m_pixmap->height())/2);
+  m_ignoreColorValue = m_image.pixelColor(x, y);
+  swatch.fill(QColor(m_ignoreColorValue.redQuantum()/257,m_ignoreColorValue.greenQuantum()/257,m_ignoreColorValue.blueQuantum()/257));
+  Import_IgnoreColorValue->setPixmap(swatch);
+  kdDebug() << "color at X:" << x << " Y:" << y << " is color " << m_ignoreColorValue.redQuantum() << "," << m_ignoreColorValue.greenQuantum() << "," << m_ignoreColorValue.blueQuantum() << endl;
+  renderPixmap();
+}
+
