@@ -18,6 +18,7 @@
 
 class QPixmap;
 class QTimerEvent;
+class AlphaSelect;
 
 class ImportImageDlg : public ImportImage
 {
@@ -38,9 +39,11 @@ public:
     @return pointer to an @ref Image Image structure
   */
   Magick::Image modifiedImage() { return m_image; }
-  QString flossScheme() { return Import_FlossScheme->currentText(); }
-  double clothCount() { return Import_ClothCount->value(); }
-  bool useFractionals() { return m_useFractionals; }
+  bool ignoreColor() const { return m_ignoreColor; }
+  Magick::Color ignoreColorValue() const { return m_ignoreColorValue; }
+  QString flossScheme() const { return Import_FlossScheme->currentText(); }
+  double clothCount() const { return Import_ClothCount->value(); }
+  bool useFractionals() const { return m_useFractionals; }
 
 public slots:
 /** resets the image to the original as loaded state, called in response to the Reset button */
@@ -70,6 +73,8 @@ public slots:
 protected slots:
   virtual void accept();
   void contextHelp();
+  void pickColor();
+  void selectColor(QPoint p);
 
 protected:
   virtual void timerEvent(QTimerEvent *);
@@ -89,6 +94,9 @@ private:
   QSize         m_originalSize;
   QSize         m_preferredSize;
   bool          m_useFractionals;
+  bool          m_ignoreColor;
+  AlphaSelect*  m_alphaSelect;
+  Magick::Color m_ignoreColorValue;
   Magick::Image m_orig;
   Magick::Image m_image;
   Magick::Image m_map;
