@@ -73,6 +73,7 @@ KXStitchApp::KXStitchApp(QWidget* , const char* name)
   connect(m_view->m_editor,SIGNAL(colorMask(bool)),this,SLOT(slotColorMask(bool)));
   connect(m_view->m_editor,SIGNAL(excludeBackstitches(bool)),this,SLOT(slotExcludeBackstitches(bool)));
   connect(m_view->m_editor,SIGNAL(excludeKnots(bool)),this,SLOT(slotExcludeKnots(bool)));
+  connect(m_view->m_palette,SIGNAL(needEditViewReset()),m_view->m_editor->viewport(),SLOT(repaint()));
 
   m_view->configure();
 
@@ -103,9 +104,10 @@ void KXStitchApp::initActions()
   new KAction(i18n("Scan Image..."), 0, this, SLOT(slotFileScan()), actionCollection(), "file_scan");
   KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
   connect(kapp, SIGNAL(lastWindowClosed()), kapp, SLOT(quit()));
-  m_viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
-  m_viewStitchBar = new KToggleAction(i18n("Show Stitch Toolbar"),0,this,SLOT(slotViewStitchBar()),actionCollection(),"viewstitchbar");
-  m_viewToolToolBar = new KToggleAction(i18n("Show Tool Toolbar"),0,this,SLOT(slotViewToolToolBar()),actionCollection(),"viewtooltoolbar");
+//  m_viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
+  KMainWindow::setStandardToolBarMenuEnabled(true);
+//  m_viewStitchBar = new KToggleAction(i18n("Show Stitch Toolbar"),0,this,SLOT(slotViewStitchBar()),actionCollection(),"viewstitchbar");
+//  m_viewToolToolBar = new KToggleAction(i18n("Show Tool Toolbar"),0,this,SLOT(slotViewToolToolBar()),actionCollection(),"viewtooltoolbar");
   m_viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
 
   new KAction(i18n("Calibrate colors..."), 0, this, SLOT(slotCalibrate()),actionCollection(),"calibrate");
@@ -180,13 +182,13 @@ void KXStitchApp::saveOptions()
   m_config->writeEntry("ParentSplitter", m_view->sizes());
   QSplitter *splitter = (QSplitter *)(m_view->child("ChildSplitter",0));
   m_config->writeEntry("ChildSplitter", splitter->sizes());
-  m_config->writeEntry("Show Toolbar", m_viewToolBar->isChecked());
-  m_config->writeEntry("Show StitchBar", m_viewStitchBar->isChecked());
-  m_config->writeEntry("Show ToolToolBar", m_viewToolToolBar->isChecked());
+//  m_config->writeEntry("Show Toolbar", m_viewToolBar->isChecked());
+//  m_config->writeEntry("Show StitchBar", m_viewStitchBar->isChecked());
+//  m_config->writeEntry("Show ToolToolBar", m_viewToolToolBar->isChecked());
   m_config->writeEntry("Show Statusbar",m_viewStatusBar->isChecked());
-  m_config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
-  m_config->writeEntry("StitchToolBarPos", (int) toolBar("StitchToolBar")->barPos());
-  m_config->writeEntry("ToolToolBarPos", (int) toolBar("ToolsToolbar")->barPos());
+//  m_config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
+//  m_config->writeEntry("StitchToolBarPos", (int) toolBar("StitchToolBar")->barPos());
+//  m_config->writeEntry("ToolToolBarPos", (int) toolBar("ToolsToolbar")->barPos());
   m_fileOpenRecent->saveEntries(m_config,"Recent Files");
 }
 
@@ -195,30 +197,30 @@ void KXStitchApp::readOptions()
   m_config->setGroup("General Options");
 
   // bar status settings
-  bool bViewToolbar = m_config->readBoolEntry("Show Toolbar", true);
-  m_viewToolBar->setChecked(bViewToolbar);
-  slotViewToolBar();
+//  bool bViewToolbar = m_config->readBoolEntry("Show Toolbar", true);
+//  m_viewToolBar->setChecked(bViewToolbar);
+//  slotViewToolBar();
 
-  bool bViewStitchBar = m_config->readBoolEntry("Show StitchBar", true);
-  m_viewStitchBar->setChecked(bViewStitchBar);
-  slotViewStitchBar();
+//  bool bViewStitchBar = m_config->readBoolEntry("Show StitchBar", true);
+//  m_viewStitchBar->setChecked(bViewStitchBar);
+//  slotViewStitchBar();
 
-  bool bViewToolToolBar = m_config->readBoolEntry("Show ToolToolBar", true);
-  m_viewToolToolBar->setChecked(bViewToolToolBar);
-  slotViewToolToolBar();
+//  bool bViewToolToolBar = m_config->readBoolEntry("Show ToolToolBar", true);
+//  m_viewToolToolBar->setChecked(bViewToolToolBar);
+//  slotViewToolToolBar();
 
   bool bViewStatusbar = m_config->readBoolEntry("Show Statusbar", true);
   m_viewStatusBar->setChecked(bViewStatusbar);
   slotViewStatusBar();
 
   // bar position settings
-  KToolBar::BarPosition toolBarPos;
-  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("ToolBarPos", KToolBar::Top);
-  toolBar("mainToolBar")->setBarPos(toolBarPos);
-  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("StitchToolBarPos", KToolBar::Top);
-  toolBar("StitchToolbar")->setBarPos(toolBarPos);
-  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("ToolToolBarPos", KToolBar::Top);
-  toolBar("ToolsToolbar")->setBarPos(toolBarPos);
+//  KToolBar::BarPosition toolBarPos;
+//  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("ToolBarPos", KToolBar::Top);
+//  toolBar("mainToolBar")->setBarPos(toolBarPos);
+//  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("StitchToolBarPos", KToolBar::Top);
+//  toolBar("StitchToolbar")->setBarPos(toolBarPos);
+//  toolBarPos=(KToolBar::BarPosition) m_config->readNumEntry("ToolToolBarPos", KToolBar::Top);
+//  toolBar("ToolsToolbar")->setBarPos(toolBarPos);
 
   // initialize the recent file list
   m_fileOpenRecent->loadEntries(m_config,"Recent Files");
@@ -492,6 +494,7 @@ void KXStitchApp::slotFileQuit()
 
 void KXStitchApp::slotViewToolBar()
 {
+/*
   slotStatusMsg(i18n("Toggling toolbar..."));
   if(!m_viewToolBar->isChecked())
   {
@@ -502,6 +505,7 @@ void KXStitchApp::slotViewToolBar()
     toolBar("mainToolBar")->show();
   }
   slotStatusMsg(i18n("Ready."));
+*/
 }
 
 void KXStitchApp::slotViewStitchBar()
