@@ -63,6 +63,8 @@ KXStitchDoc::KXStitchDoc(QWidget *parent, const char *name)
   m_palette = new FlossPalette();
   m_canvas = new PatternCanvas();
   m_preview = new QPixmap();
+  m_undoStack.setAutoDelete(true);
+  m_redoStack.setAutoDelete(true);
   m_docURL.setFileName(i18n("Untitled"));
   setModified(false);
 }
@@ -241,12 +243,12 @@ void KXStitchDoc::fileProperties()
           requestedSize.setHeight(currentRect.height());
         if (requestedSize.width() > currentRect.width())
         {
-          l = (requestedSize.width()-currentRect.width())<?currentRect.left();
+          l = std::min((requestedSize.width()-currentRect.width()), currentRect.left());
           r = requestedSize.width()-(currentRect.width()+l);
         }
         if (requestedSize.height() > currentRect.height())
         {
-          t = (requestedSize.height()-currentRect.height())<?currentRect.top();
+          t = std::min((requestedSize.height()-currentRect.height()), currentRect.top());
           b = requestedSize.height()-(currentRect.height()+t);
         }
         cropCanvasToPattern();
@@ -283,6 +285,10 @@ void KXStitchDoc::fileProperties()
 
 bool KXStitchDoc::newDocument()
 {
+  m_undoStack.clear();
+  emit undoStackNotEmpty(false);
+  m_redoStack.clear();
+  emit redoStackNotEmpty(false);
   m_propertiesDialog->reset();
   m_docURL.setFileName(i18n("Untitled"));
   m_palette->clear();
@@ -378,7 +384,7 @@ void KXStitchDoc::updatePixmap(QRect rect)
   }
   p->end();
 }
-
+/*
 bool KXStitchDoc::readKXStitchDocument(QDataStream& s)
 {
   QString magic;
@@ -440,7 +446,7 @@ bool KXStitchDoc::readKXStitchDocument(QDataStream& s)
   }
   return validRead;
 }
-
+*/
 bool KXStitchDoc::readPCStitch5Document(QDataStream& s)
 {
   char*   buffer = new char[256];
@@ -516,31 +522,33 @@ bool KXStitchDoc::openDocument(const KURL& url, const char*)
   deleteContents();
   if (!url.isEmpty())
   {
-    KIO::NetAccess::download(url,tmpfile);
-    QFile f(tmpfile);
-    if (f.open(IO_ReadOnly))
+    if (KIO::NetAccess::download(url, tmpfile, 0))
     {
-      QDataStream s(&f);
-      // Check first byte of file, which might indicate certain formats to try
-      // 'P' may be a PCStitch file version 5 or later
-      // 0 may be a KXStitch file
-      s >> magic;
-      f.reset();
-      switch (magic)
+      QFile f(tmpfile);
+      if (f.open(IO_ReadOnly))
       {
-        case 'P':
-          validRead = readPCStitch5Document(s);
-          break;
-        case '\0':
-          validRead = readKXStitchDocument(s);
-          break;
-        default:
-          validRead = false;
-          break;
+        QDataStream s(&f);
+        // Check first byte of file, which might indicate certain formats to try
+        // 'P' may be a PCStitch file version 5 or later
+        // 0 may be a KXStitch file
+        s >> magic;
+        f.reset();
+        switch (magic)
+        {
+          case 'P':
+            validRead = readPCStitch5Document(s);
+            break;
+          case '\0':
+            validRead = readKXStitchStream(s);
+            break;
+          default:
+            validRead = false;
+            break;
+        }
       }
+      f.close();
+      KIO::NetAccess::removeTempFile( tmpfile );
     }
-    f.close();
-    KIO::NetAccess::removeTempFile( tmpfile );
   }
   if (validRead)
     setURL(url);
@@ -563,12 +571,19 @@ bool KXStitchDoc::importImage()
   url = KFileDialog::getImageOpenURL(QString::null,(KXStitchApp *)parent(),i18n("Import Image"));
   if (!url.path().isNull())
   {
-    KIO::NetAccess::download(url,tmpfile);
-    Magick::Image magickImage(tmpfile);
-    convertImage(magickImage);
-    m_propertiesDialog->setTitle(url.fileName());
-    KIO::NetAccess::removeTempFile(tmpfile);
-    return true;
+    if (KIO::NetAccess::download(url, tmpfile, 0))
+    {
+      Magick::Image magickImage((std::string)(const char*)tmpfile);
+      convertImage(magickImage);
+      m_propertiesDialog->setTitle(url.fileName());
+      KIO::NetAccess::removeTempFile(tmpfile);
+      return true;
+    }
+    else
+    {
+      KMessageBox::sorry(0,i18n("KXStitch could not import the file specified."),i18n("File import error"));
+      newDocument();
+    }
   }
   return false;
 }
@@ -620,6 +635,9 @@ void KXStitchDoc::convertImage(const Magick::Image& magickImage)
     Magick::Image image = importDlg->modifiedImage();
     int w = image.columns();
     int h = image.rows();
+    bool ignoreColor = importDlg->ignoreColor();
+    Magick::Color ignoreColorValue = importDlg->ignoreColorValue();
+    kdDebug() << "ignore color " << ignoreColorValue.redQuantum() << "," << ignoreColorValue.greenQuantum() << "," << ignoreColorValue.blueQuantum() << endl;
     m_palette->setScheme(m_propertiesDialog->scheme());
     int pixelCount = w*h;
     if (importDlg->useFractionals())
@@ -632,7 +650,7 @@ void KXStitchDoc::convertImage(const Magick::Image& magickImage)
       QProgressDialog progress(i18n("Converting to stitches..."), i18n("Cancel"), pixelCount, (KXStitchApp *)parent(), i18n("Progress"), true);
       progress.setMinimumDuration(0);
       Magick::Pixels cache(image);
-      const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
+      const Magick::PixelPacket *pixels = cache.getConst(0,0,w,h);
       for (int cy = 0 ; cy < h ; cy++)
       {
         progress.setProgress(cy*w);
@@ -641,23 +659,26 @@ void KXStitchDoc::convertImage(const Magick::Image& magickImage)
         for (int cx = 0 ; cx < w ; cx++)
         {
           Magick::PixelPacket packet = *pixels++;
-          if (!packet.opacity)
+          if (!(packet.opacity))
           {
-            QColor c(packet.red/257, packet.green/257, packet.blue/257);
-            i = m_palette->addColor(c);
-            if (cx % 2)
+            if (!(ignoreColor && packet == ignoreColorValue))
             {
-              if (cy % 2)
-                addStitch(cx/2, cy/2, Stitch::BRQtr, i);
+              QColor c(packet.red/257, packet.green/257, packet.blue/257);
+              i = m_palette->addColor(c);
+              if (cx % 2)
+              {
+                if (cy % 2)
+                  addStitch(cx/2, cy/2, Stitch::BRQtr, i);
+                else
+                  addStitch(cx/2, cy/2, Stitch::TRQtr, i);
+              }
               else
-                addStitch(cx/2, cy/2, Stitch::TRQtr, i);
-            }
-            else
-            {
-              if (cy % 2)
-                addStitch(cx/2, cy/2, Stitch::BLQtr, i);
-              else
-                addStitch(cx/2, cy/2, Stitch::TLQtr, i);
+              {
+                if (cy % 2)
+                  addStitch(cx/2, cy/2, Stitch::BLQtr, i);
+                else
+                  addStitch(cx/2, cy/2, Stitch::TLQtr, i);
+              }
             }
           }
         }
@@ -671,7 +692,7 @@ void KXStitchDoc::convertImage(const Magick::Image& magickImage)
       QProgressDialog progress(i18n("Converting to stitches..."), i18n("Cancel"), pixelCount, (KXStitchApp *)parent(), i18n("Progress"), true);
       progress.setMinimumDuration(0);
       Magick::Pixels cache(image);
-      const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
+      const Magick::PixelPacket *pixels = cache.getConst(0,0,w,h);
       for (int cy = 0 ; cy < h ; cy++)
       {
         progress.setProgress(cy*w);
@@ -682,9 +703,12 @@ void KXStitchDoc::convertImage(const Magick::Image& magickImage)
           Magick::PixelPacket packet = *pixels++;
           if (!packet.opacity)
           {
-            QColor c(packet.red/257, packet.green/257, packet.blue/257);
-            i = m_palette->addColor(c);
-            addStitch(cx,cy,Stitch::Full,i);
+            if (!(ignoreColor && packet == ignoreColorValue))
+            {
+              QColor c(packet.red/257, packet.green/257, packet.blue/257);
+              i = m_palette->addColor(c);
+              addStitch(cx,cy,Stitch::Full,i);
+            }
           }
         }
       }
@@ -709,6 +733,67 @@ bool KXStitchDoc::saveDocument(const KURL& url, const char*)
   return true;
 }
 
+bool KXStitchDoc::readKXStitchStream(QDataStream& s)
+{
+  QString magic;
+  Q_INT16 version;
+  bool    validRead = false;
+  QString title;
+  QString author;
+  QString fabric;
+  QColor  fabricColor;
+  Q_INT16 scheme;
+  Q_INT16 inches;
+  Q_INT32 width;
+  Q_INT32 height;
+  Q_INT32 clothCount;
+  QString instructions;
+  s >> magic;
+  if (magic == "KXStitch")
+  {
+    s >> version;
+    switch (version)
+    {
+      case 6: // no change for this section
+      case 5:
+        s >> *m_propertiesDialog;
+        validRead = m_palette->readKXStitchPalette(s,version) && m_canvas->readKXStitchCanvas(s,version);
+        break;
+      case 4:
+      case 3:
+      case 2:
+        s >> title >> author >> fabric >> fabricColor >> scheme >> inches >> width >> height >> clothCount >> instructions;
+        m_propertiesDialog->setTitle(title);
+        m_propertiesDialog->setAuthor(author);
+        m_propertiesDialog->setCopyright(QString());
+        m_propertiesDialog->setFabric(fabric);
+        m_propertiesDialog->setFabricColor(fabricColor);
+        m_propertiesDialog->setUnits(bool(inches)?"Inches":"Stitches");
+        m_propertiesDialog->setClothCount(double(clothCount));
+        m_propertiesDialog->setClothCountUnits("/in");
+        if (m_propertiesDialog->units() == "Inches")
+        {
+          width *= clothCount;
+          height *= clothCount;
+        }
+        m_propertiesDialog->setPatternSize(width,height);
+        m_propertiesDialog->setInstructions(instructions);
+        validRead = m_palette->readKXStitchPalette(s,version);
+        if (validRead)
+        {
+          m_propertiesDialog->setScheme(m_palette->getScheme());
+          validRead = m_canvas->readKXStitchCanvas(s,version);
+        }
+        break;
+      case 1:
+      default:
+        KMessageBox::sorry(0, i18n("The version of this pattern file is not supported by this version of the software."), i18n("Unsupported File Format"));
+        break;
+    }
+  }
+  return validRead;
+}
+
 void KXStitchDoc::writeKXStitchStream(QDataStream &s)
 {
   // write file magic number and format version
@@ -728,9 +813,10 @@ void KXStitchDoc::createSavePoint()
   QByteArray *b = new QByteArray();
   *b = qCompress(a);
   m_undoStack.push(b);
+  emit undoStackNotEmpty(true);
 }
 
-bool KXStitchDoc::undo()
+void KXStitchDoc::undo()
 {
   createSavePoint();
   m_redoStack.push(m_undoStack.pop());
@@ -740,14 +826,15 @@ bool KXStitchDoc::undo()
   m_canvas->clear();
   QByteArray *a = m_undoStack.pop();
   QDataStream s(qUncompress(*a), IO_ReadOnly);
-  readKXStitchDocument(s);
+  readKXStitchStream(s);
   renderPixmap();
   m_docURL = url;
   slotResetAllViews();
-  return !m_undoStack.isEmpty();
+  emit undoStackNotEmpty(!m_undoStack.isEmpty());
+  emit redoStackNotEmpty(!m_redoStack.isEmpty());
 }
 
-bool KXStitchDoc::redo()
+void KXStitchDoc::redo()
 {
   createSavePoint();
   KURL url = m_docURL;
@@ -756,11 +843,11 @@ bool KXStitchDoc::redo()
   m_canvas->clear();
   QByteArray *a = m_redoStack.pop();
   QDataStream s(qUncompress(*a), IO_ReadOnly);
-  readKXStitchDocument(s);
+  readKXStitchStream(s);
   renderPixmap();
   m_docURL = url;
   slotResetAllViews();
-  return !m_redoStack.isEmpty();
+  emit redoStackNotEmpty(!m_redoStack.isEmpty());
 }
 
 void KXStitchDoc::deleteContents()
