@@ -20,7 +20,7 @@
 #include <kaction.h>
 #include <kprinter.h>
 #include <kiconloader.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 #include "configuration.h"
@@ -181,8 +181,6 @@ PaletteView::PaletteView(KXStitchDoc* doc,QWidget *parent)
 {
   KIconLoader* loader = KGlobal::iconLoader();
   KActionCollection* actionCollection = ((KXStitchApp*)topLevelWidget())->actionCollection();
-  new KAction(i18n("Add color to palette..."), QIconSet(loader->loadIcon("appearance",KIcon::User)), 0, this, SLOT(slotAddColor()), actionCollection, "addcolor");
-  new KAction(i18n("Remove colors from palette..."), 0, this, SLOT(slotRemoveColors()), actionCollection, "removecolors");
   m_savePalette = new KAction(i18n("Save the current palette..."), 0, this, SLOT(slotSavePalette()), actionCollection, "savepalette");
   m_loadPalette = new KAction(i18n("Load a palette..."), 0, this, SLOT(slotLoadPalette()), actionCollection, "loadpalette");
   m_createPalette = new KAction(i18n("Create a palette..."), 0, this, SLOT(slotCreatePalette()), actionCollection, "createpalette");
@@ -190,6 +188,7 @@ PaletteView::PaletteView(KXStitchDoc* doc,QWidget *parent)
   new KAction(i18n("Clear unused colors"), 0, this, SLOT(slotClearUnused()), actionCollection, "clearunused");
   new KAction(i18n("Swap colors"), 0, this, SLOT(slotPrepareToSwap()), actionCollection, "swapcolors");
   new KAction(i18n("Replace color"), 0, this, SLOT(slotPrepareToReplace()), actionCollection, "replacecolor");
+  new KAction(i18n("Palette Manager"), QIconSet(loader->loadIcon("appearance.png",KIcon::User)), 0, this, SLOT(slotPaletteManager()), actionCollection, "palettemanager");
 }
 
 PaletteView::~PaletteView()
@@ -201,11 +200,7 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
 {
   FlossPalette *palette = m_doc->palette();
   UsageMap usageMap = m_doc->flossUsage();
-  int stitchStrands = KXSConfig().Palette_StitchStrands;
-  int backstitchStrands = KXSConfig().Palette_BackstitchStrands;
   int flossStrands = KXSConfig().Palette_FlossStrands;
-  double stitchLengthDivisor = (double)flossStrands/(double)stitchStrands;
-  double backstitchLengthDivisor = (double)flossStrands/(double)backstitchStrands;
   QString paletteName = QString(i18n("%1 Flosses")).arg(palette->getScheme());
   QFont font = painter->font();
   QFont symbols = font;
@@ -218,8 +213,18 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
 
   int wSymbol = 0;
   int wName = 0;
+  int wStrands = 0;
   int wDescription = 0;
   int wLength = 0;
+
+  int fullTotal = 0;
+  int halfTotal = 0;
+  int quarterTotal = 0;
+  int threequarterTotal = 0;
+  int backstitchTotal = 0;
+  int knotTotal = 0;
+  int fractionalsTotal = 0;
+
   QValueVector<int>::size_type floss;
   for (floss = 0 ; floss < m_flosses.count() ; floss++)
   {
@@ -235,21 +240,29 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
   painter->setFont(font);
   fm = painter->fontMetrics();
   if (fm.width(i18n("Symbol"))>wSymbol)
-    wSymbol = fm.width("Symbol");
+    wSymbol = fm.width(i18n("Symbol"));
   if (fm.width(i18n("Name"))>wName)
-    wName = fm.width("Name");
+    wName = fm.width(i18n("Name"));
+  wStrands = fm.width(i18n("Strands"));
   if (fm.width(i18n("Description"))>wDescription)
-    wDescription = fm.width("Description");
-  if (fm.width(i18n("Length(m)"))>wLength)
-    wLength = fm.width(i18n("Length(m)"));
+    wDescription = fm.width(i18n("Description"));
+  wLength = fm.width(i18n("Length(m)"));
 
   // widen the fields to allow some spacing
   wSymbol += 20;
   wName += 20;
+  wStrands += 20;
   wDescription += 20;
 
   for (floss = 0 ; floss < m_flosses.count() ; floss++)
   {
+    fullTotal += usageMap[m_flosses[floss]].full;
+    halfTotal += usageMap[m_flosses[floss]].half;
+    quarterTotal += usageMap[m_flosses[floss]].quarter;
+    threequarterTotal += usageMap[m_flosses[floss]].threeQuarter;
+    backstitchTotal += usageMap[m_flosses[floss]].backstitches;
+    knotTotal += usageMap[m_flosses[floss]].knots;
+
     if (y == dy)
     {
       font.setBold(true);
@@ -258,7 +271,7 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
       y += dy*2;
       if (printer->option("kde-kxstitch-flossusage") == "true")
       {
-        painter->drawText(0,y,QString(i18n("Lengths shown assume %1 strands of a %2 strand skein are used for stitching and %3 for backstitching.")).arg(stitchStrands).arg(flossStrands).arg(backstitchStrands));
+        painter->drawText(0,y,QString(i18n("Strand numbers shown are for stitches / backstitches.")));
         if (printer->option("kde-kxstitch-seperatebackstitching") == "true")
           painter->drawText(0,y+dy,QString(i18n("Lengths are for Stitches, Backstitches (Total).")));
         else
@@ -275,10 +288,11 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
       }
       painter->drawText(0,y,i18n("Symbol"));
       painter->drawText(wSymbol,y,i18n("Name"));
-      painter->drawText(wSymbol+wName,y,i18n("Description"));
+      painter->drawText(wSymbol+wName,y,i18n("Strands"));
+      painter->drawText(wSymbol+wName+wStrands,y,i18n("Description"));
       if (printer->option("kde-kxstitch-flossusage") == "true")
       {
-        painter->drawText(wSymbol+wName+wDescription,y,i18n("Length(m)"));
+        painter->drawText(wSymbol+wName+wStrands+wDescription,y,i18n("Length(m)"));
       }
       font.setBold(false);
       painter->setFont(font);
@@ -286,43 +300,50 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
     }
     Floss *f = palette->flossAt(m_flosses[floss]);
     QChar s = palette->symbolAt(m_flosses[floss]);
+    int stitchStrands = palette->stitchStrandsAt(m_flosses[floss]);
+    int backstitchStrands = palette->backstitchStrandsAt(m_flosses[floss]);
+    double stitchLengthDivisor = (double)flossStrands/(double)stitchStrands;
+    double backstitchLengthDivisor = (double)flossStrands/(double)backstitchStrands;
     /** set the font for the symbol */
     painter->setFont(symbols);
     painter->drawText(0,y,s);
     /** set the font for the text */
     painter->setFont(font);
     painter->drawText(wSymbol,y,f->name);
-    painter->drawText(wSymbol+wName,y,f->description);
+    painter->drawText(wSymbol+wName,y,QString("%1 / %2").arg(stitchStrands).arg(backstitchStrands));
+    painter->drawText(wSymbol+wStrands+wName,y,f->description);
     if (printer->option("kde-kxstitch-flossusage") == "true")
     {
       double combinedLength = usageMap[m_flosses[floss]].stitchLength/stitchLengthDivisor + usageMap[m_flosses[floss]].backstitchLength/backstitchLengthDivisor;
       int fieldWidth = fm.width("99999 ");
       int lengthsFieldWidth = fieldWidth*3;
       if (printer->option("kde-kxstitch-seperatebackstitching") == "true")
-        painter->drawText(wSymbol+wName+wDescription,y,QString("%1 %2 (%3)").arg(usageMap[m_flosses[floss]].stitchLength/stitchLengthDivisor,0,'f',2).arg(usageMap[m_flosses[floss]].backstitchLength/backstitchLengthDivisor,0,'f',2).arg(combinedLength,0,'f',2));
+        painter->drawText(wSymbol+wStrands+wName+wDescription,y,QString("%1 %2 (%3)").arg(usageMap[m_flosses[floss]].stitchLength/stitchLengthDivisor,0,'f',2).arg(usageMap[m_flosses[floss]].backstitchLength/backstitchLengthDivisor,0,'f',2).arg(combinedLength,0,'f',2));
       else
-        painter->drawText(wSymbol+wName+wDescription,y,QString("%1").arg(combinedLength,0,'f',2));
+        painter->drawText(wSymbol+wStrands+wName+wDescription,y,QString("%1").arg(combinedLength,0,'f',2));
       if (printer->option("kde-kxstitch-stitchcount") == "true")
       {
         if (printer->option("kde-kxstitch-combinefractionals") == "true")
         {
           int fractionals = usageMap[m_flosses[floss]].half + usageMap[m_flosses[floss]].quarter + usageMap[m_flosses[floss]].threeQuarter;
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth,y,QString("%1").arg(usageMap[m_flosses[floss]].full));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth,y,QString("%1").arg(fractionals));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth*2,y,QString("%1").arg(usageMap[m_flosses[floss]].knots));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth*3,y,QString("%1").arg(usageMap[m_flosses[floss]].backstitches));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth,y,QString("%1").arg(usageMap[m_flosses[floss]].full));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth,y,QString("%1").arg(fractionals));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*2,y,QString("%1").arg(usageMap[m_flosses[floss]].knots));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*3,y,QString("%1").arg(usageMap[m_flosses[floss]].backstitches));
         }
         else
         {
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth,y,QString("%1").arg(usageMap[m_flosses[floss]].full));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth,y,QString("%1").arg(usageMap[m_flosses[floss]].half));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth*2,y,QString("%1").arg(usageMap[m_flosses[floss]].quarter));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth*3,y,QString("%1").arg(usageMap[m_flosses[floss]].threeQuarter));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth*4,y,QString("%1").arg(usageMap[m_flosses[floss]].knots));
-          painter->drawText(wSymbol+wName+wDescription+lengthsFieldWidth+fieldWidth*5,y,QString("%1").arg(usageMap[m_flosses[floss]].backstitches));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth,y,QString("%1").arg(usageMap[m_flosses[floss]].full));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth,y,QString("%1").arg(usageMap[m_flosses[floss]].half));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*2,y,QString("%1").arg(usageMap[m_flosses[floss]].quarter));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*3,y,QString("%1").arg(usageMap[m_flosses[floss]].threeQuarter));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*4,y,QString("%1").arg(usageMap[m_flosses[floss]].knots));
+          painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*5,y,QString("%1").arg(usageMap[m_flosses[floss]].backstitches));
         }
       }
     }
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
     y += dy;
     if (y > height)
     {
@@ -330,12 +351,107 @@ int PaletteView::print(KPrinter *printer, QPainter *painter, int width, int heig
       painter->drawText(0,height+dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(pages++));
       printer->newPage();
     }
+#endif
   }
-
-  painter->drawText(0,height+dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(pages));
+  fractionalsTotal = halfTotal + quarterTotal + threequarterTotal;
+  if (printer->option("kde-kxstitch-flossusage") == "true")
+  {
+    int fieldWidth = fm.width("99999 ");
+    int lengthsFieldWidth = fieldWidth*3;
+    if (printer->option("kde-kxstitch-stitchtotals") == "true")
+    {
+      if (printer->option("kde-kxstitch-combinefractionals") == "true")
+      {
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth,y,QString("%1").arg(fullTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth,y,QString("%1").arg(fractionalsTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*2,y,QString("%1").arg(knotTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*3,y,QString("%1").arg(backstitchTotal));
+      }
+      else
+      {
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth,y,QString("%1").arg(fullTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth,y,QString("%1").arg(halfTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*2,y,QString("%1").arg(quarterTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*3,y,QString("%1").arg(threequarterTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*4,y,QString("%1").arg(knotTotal));
+        painter->drawText(wSymbol+wStrands+wName+wDescription+lengthsFieldWidth+fieldWidth*5,y,QString("%1").arg(backstitchTotal));
+      }
+    }
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+    if (y > height)
+    {
+      y = dy;
+      painter->drawText(0,height+dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(pages++));
+      printer->newPage();
+    }
+#endif
+  }
+  if ((printer->option("kde-kxstitch-flossusage") == "true") && (printer->option("kde-kxstitch-stitchsummary") == "true"))
+  {
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+#endif
+    font.setBold(true);
+    painter->setFont(font);
+    painter->drawText(0,y,i18n("Stitch Summary"));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+#endif
+    font.setBold(false);
+    painter->setFont(font);
+    painter->drawText(0,y,i18n("Full Stitches"));
+    painter->drawText(250,y,QString("%1").arg(fullTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+#endif
+    painter->drawText(0,y,i18n("Half Stitches"));
+    painter->drawText(250,y,QString("%1").arg(halfTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+#endif
+    painter->drawText(0,y,i18n("Quarter Stitches"));
+    painter->drawText(250,y,QString("%1").arg(quarterTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+#endif
+    painter->drawText(0,y,i18n("3 Quarter Stitches"));
+    painter->drawText(250,y,QString("%1").arg(threequarterTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+#if 0
+    y += dy;
+#endif
+    painter->drawText(0,y,i18n("Backstitches"));
+    painter->drawText(250,y,QString("%1").arg(backstitchTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+    painter->drawText(0,y,i18n("Knots"));
+    painter->drawText(250,y,QString("%1").arg(knotTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+    painter->drawText(0,y,i18n("Total Fractionals"));
+    painter->drawText(250,y,QString("%1").arg(fractionalsTotal));
+    nextLine(&y, dy, height, width, painter, printer, &pages);
+  }
+  painter->drawText(0,height-dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg(pages));
   return pages;
 }
 
+void PaletteView::nextLine(int *y, int dy, int height, int width, QPainter *painter, KPrinter *printer, int *pages)
+{
+  *y += dy;
+  if (*y > height-2*dy)
+  {
+    *y = dy;
+    painter->drawText(0,height-dy,width,dy,Qt::AlignHCenter,QString(i18n("Page %1")).arg((*pages)++));
+    printer->newPage();
+  }
+}
+
 void PaletteView::setCurrentColor(int c)
 {
   if (m_waitingToSwap)
@@ -354,6 +470,7 @@ void PaletteView::setCurrentColor(int c)
   }
   m_doc->palette()->setCurrentColor(c);
   update();
+  emit needEditViewReset();
 }
 
 void PaletteView::configure()
@@ -422,7 +539,7 @@ void PaletteView::drawContents(QPainter *p)
   {
     ((KXStitchApp*)topLevelWidget())->slotStateChanged("palette_empty");
   }
-  f.setPixelSize(m_cellWidth<?m_cellHeight);
+  f.setPixelSize(std::min(m_cellWidth, m_cellHeight));
   p->setFont(f);
   for (int i = 0 ; i < palette->colors() ; i++)
   {
@@ -510,7 +627,7 @@ void PaletteView::slotSavePalette()
 {
   QString schemeName;
   bool ok;
-  schemeName = KLineEditDlg::getText(i18n("Scheme Name"),"",&ok,this);
+  schemeName = KInputDialog::getText(i18n("Scheme Name"),i18n("Scheme Name"),"",&ok,this);
   if (ok)
   {
     if (((KXStitchApplication*)kapp)->scheme(schemeName))
@@ -540,7 +657,7 @@ void PaletteView::slotCreatePalette()
   bool    ok = false;
   while (!ok)
   {
-    schemeName = KLineEditDlg::getText(i18n("Scheme Name"),"",&ok,this);
+    schemeName = KInputDialog::getText(i18n("Scheme Name"),i18n("Scheme Name"),"",&ok,this);
     if (!ok)
       break; // user pressed cancel
     if (((KXStitchApplication*)kapp)->scheme(schemeName))
@@ -583,17 +700,21 @@ void PaletteView::slotPrepareToReplace()
 
 void PaletteView::slotAddColor()
 {
+/*
   if (m_doc->palette()->addColors())
   {
     update();
   }
+*/
 }
 void PaletteView::slotRemoveColors()
 {
+/*
   if (m_doc->removeColors())
   {
     update();
   }
+*/
 }
 
 void PaletteView::slotClearUnused()
@@ -601,3 +722,11 @@ void PaletteView::slotClearUnused()
   m_doc->removeUnusedColors();
   update();
 }
+
+void PaletteView::slotPaletteManager()
+{
+  if (m_doc->paletteManager())
+  {
+    update();
+  }
+}
