@@ -143,10 +143,10 @@ QRect PatternCanvas::patternBoundingRect()
     int y2 = 0;
     for (Knot *k = m_pKnot->first() ; k ; k = m_pKnot->next())
     {
-      x1 = x1 <? k->pos.x();
-      y1 = y1 <? k->pos.y();
-      x2 = x2 >? k->pos.x();
-      y2 = y2 >? k->pos.y();
+      x1 = std::min(x1, k->pos.x());
+      y1 = std::min(y1, k->pos.y());
+      x2 = std::max(x2, k->pos.x());
+      y2 = std::max(y2, k->pos.y());
     }
     stitchesRect|=QRect(x1/2,y1/2,(x2-x1)/2,(y2-y1)/2);
   }
@@ -159,10 +159,10 @@ QRect PatternCanvas::patternBoundingRect()
       QPoint s = bs->start;
       QPoint e = bs->end;
       int x1,y1,x2,y2;
-      x1 = s.x() <? e.x();
-      x2 = s.x() >? e.x();
-      y1 = s.y() <? e.y();
-      y2 = s.y() >? e.y();
+      x1 = std::min(s.x(), e.x());
+      x2 = std::max(s.x(), e.x());
+      y1 = std::min(s.y(), e.y());
+      y2 = std::max(s.y(), e.y());
       if (x1 == x2 || y1 == y2)
       {
         // special case as a QRect would show as zero m_width or m_height (ie invalid)
